import 'package:utilary/utilary.dart';
import 'package:chess_logic/chess_logic.dart';

///Represents a single tile on a chess board:
///A letter from `a` to `h` is used to address
///a column, called a file,  on the board. To address a row
///which is called rank,
///a number from `1` to `8` is used.
///(e.g. tile `e4` is mapped to `Vector(4, 3)`)
class ChessPosition extends Vector {
  final String file;
  final String rank;

  factory ChessPosition(Object pos) {
    if (pos is String) {
      //Make sure provided pos as string is valid
      if (RegExp(r"^[a-hA-H][1-8]").hasMatch(pos)) {
        return ChessPosition._(pos[0], pos[1]);
      }

      //Provided pos is invalid
      throw ArgumentError("Invalid string argument");
    }

    if (pos is Vector) {
      //Make sure provided pos as Vector is valid
      if (Rectangle(0, 0, 8, 8).contains(pos)) {
        return ChessPosition.fromVector(pos);
      }

      //invalid pos
      throw ArgumentError("Invalid vector argument");
    }

    throw ArgumentError("Argument must be of type String or Vector");
  }

  ChessPosition._(this.file, this.rank)
      : super(ArithmeticNotation.fileToIndex(file),
            ArithmeticNotation.rankToIndex(int.parse(rank)));

  ChessPosition.fromVector(Vector pos)
      : file = ArithmeticNotation.indexToFile(pos.x),
        rank = ArithmeticNotation.indexToRank(pos.y).toString(),
        super(pos.x, pos.y);



  String toString() => file + rank;
}
